@@ -471,13 +471,12 @@ thunar_sbr_number_renamer_update (ThunarSbrNumberRenamer *number_renamer)
   GdkColor back;
   GdkColor text;
   gchar   *endp;
-  guint    number;
 
   /* check whether "start" is valid for the "mode" */
   if (number_renamer->mode < THUNAR_SBR_NUMBER_MODE_ABC)
     {
       /* "start" must be a positive number */
-      number = strtoul (number_renamer->start, &endp, 10);
+      strtoul (number_renamer->start, &endp, 10);
       invalid = (endp <= number_renamer->start || *endp != '\0');
     }
   else if (number_renamer->mode == THUNAR_SBR_NUMBER_MODE_ABC)
@@ -489,7 +488,7 @@ thunar_sbr_number_renamer_update (ThunarSbrNumberRenamer *number_renamer)
     }
 
   /* check if the start entry is realized */
-  if (GTK_WIDGET_REALIZED (number_renamer->start_entry))
+  if (gtk_widget_get_realized (number_renamer->start_entry))
     {
       /* check if the "start" value is valid */
       if (G_UNLIKELY (invalid))
