@@ -1,25 +1,22 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
- * Copyright (c) 2009-2010 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * This program is free software; you can redistribute it and/or 
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of 
+ * the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * The basic idea for the icon factory implementation was borrowed from
- * Nautilus initially, but the implementation is very different from
- * what Nautilus does.
+ * You should have received a copy of the GNU General Public 
+ * License along with this program; if not, write to the Free 
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,15 +35,11 @@
 #include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-thumbnail-frame.h>
-#include <thunar/thunar-thumbnailer.h>
 
 
 
-/* the timeout until the sweeper is run (in ms) */
-#define THUNAR_ICON_FACTORY_SWEEP_TIMEOUT (30 * 1000)
-
-/* the maximum length of the recently used list */
-#define MAX_RECENTLY (32u)
+/* the timeout until the sweeper is run (in seconds) */
+#define THUNAR_ICON_FACTORY_SWEEP_TIMEOUT (30)
 
 
 
@@ -55,7 +48,7 @@ enum
 {
   PROP_0,
   PROP_ICON_THEME,
-  PROP_SHOW_THUMBNAILS,
+  PROP_THUMBNAIL_MODE,
 };
 
 
@@ -87,11 +80,10 @@ static GdkPixbuf *thunar_icon_factory_lookup_icon           (ThunarIconFactory
                                                              const gchar              *name,
                                                              gint                      size,
                                                              gboolean                  wants_default);
-static void       thunar_icon_factory_mark_recently_used    (ThunarIconFactory        *factory,
-                                                             GdkPixbuf                *pixbuf);
 static guint      thunar_icon_key_hash                      (gconstpointer             data);
 static gboolean   thunar_icon_key_equal                     (gconstpointer             a,
                                                              gconstpointer             b);
+static void       thunar_icon_key_free                      (gpointer                  data);
 static GdkPixbuf *thunar_icon_factory_load_fallback         (ThunarIconFactory        *factory,
                                                              gint                      size);
 
@@ -106,23 +98,20 @@ struct _ThunarIconFactory
 {
   GObject __parent__;
 
-  ThunarThumbnailer *thumbnailer;
+  ThunarPreferences   *preferences;
 
-  ThunarPreferences *preferences;
+  GHashTable          *icon_cache;
 
-  GdkPixbuf         *recently[MAX_RECENTLY];  /* ring buffer */
-  guint              recently_pos;            /* insert position */
+  GtkIconTheme        *icon_theme;
 
-  GHashTable        *icon_cache;
+  ThunarThumbnailMode  thumbnail_mode;
 
-  GtkIconTheme      *icon_theme;
+  guint                sweep_timer_id;
 
-  gboolean           show_thumbnails;
+  gulong               changed_hook_id;
 
-  gint               changed_idle_id;
-  gint               sweep_timer_id;
-
-  gulong             changed_hook_id;
+  /* stamp that gets bumped when the theme changes */
+  guint                theme_stamp;
 };
 
 struct _ThunarIconKey
@@ -131,12 +120,20 @@ struct _ThunarIconKey
   gint   size;
 };
 
+typedef struct
+{
+  ThunarFileIconState   icon_state;
+  ThunarFileThumbState  thumb_state;
+  gint                  icon_size;
+  guint                 stamp;
+  GdkPixbuf            *icon;
+}
+ThunarIconStore;
+
 
 
 static GQuark thunar_icon_factory_quark = 0;
-static GQuark thunar_icon_thumb_uri_quark = 0;
-static GQuark thunar_icon_thumb_time_quark = 0;
-static GQuark thunar_file_thumb_uri_quark = 0;
+static GQuark thunar_icon_factory_store_quark = 0;
 
 
 
@@ -149,12 +146,7 @@ thunar_icon_factory_class_init (ThunarIconFactoryClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* setup the thunar-icon-thumb-{path,time} quarks */
-  thunar_icon_thumb_uri_quark = g_quark_from_static_string ("thunar-icon-thumb-path");
-  thunar_icon_thumb_time_quark = g_quark_from_static_string ("thunar-icon-thumb-time");
-
-  /* setup the thunar-file-thumb-path quark */
-  thunar_file_thumb_uri_quark = g_quark_from_static_string ("thunar-file-thumb-path");
+  thunar_icon_factory_store_quark = g_quark_from_static_string ("thunar-icon-factory-store");
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = thunar_icon_factory_dispose;
@@ -177,18 +169,19 @@ thunar_icon_factory_class_init (ThunarIconFactoryClass *klass)
                                                         EXO_PARAM_READABLE));
 
   /**
-   * ThunarIconFactory:show-thumbnails:
+   * ThunarIconFactory:thumbnail-mode:
    *
    * Whether this #ThunarIconFactory will try to generate and load thumbnails
    * when loading icons for #ThunarFile<!---->s.
    **/
   g_object_class_install_property (gobject_class,
-                                   PROP_SHOW_THUMBNAILS,
-                                   g_param_spec_boolean ("show-thumbnails",
-                                                         "show-thumbnails",
-                                                         "show-thumbnails",
-                                                         FALSE,
-                                                         EXO_PARAM_READWRITE));
+                                   PROP_THUMBNAIL_MODE,
+                                   g_param_spec_enum ("thumbnail-mode",
+                                                      "thumbnail-mode",
+                                                      "thumbnail-mode",
+                                                      THUNAR_TYPE_THUMBNAIL_MODE,
+                                                      THUNAR_THUMBNAIL_MODE_ONLY_LOCAL,
+                                                      EXO_PARAM_READWRITE));
 }
 
 
@@ -196,9 +189,7 @@ thunar_icon_factory_class_init (ThunarIconFactoryClass *klass)
 static void
 thunar_icon_factory_init (ThunarIconFactory *factory)
 {
-  /* initialize GSource ids */
-  factory->changed_idle_id = -1;
-  factory->sweep_timer_id = -1;
+  factory->thumbnail_mode = THUNAR_THUMBNAIL_MODE_ONLY_LOCAL;
 
   /* connect emission hook for the "changed" signal on the GtkIconTheme class. We use the emission
    * hook way here, because that way we can make sure that the icon cache is definetly cleared
@@ -208,10 +199,8 @@ thunar_icon_factory_init (ThunarIconFactory *factory)
                                                          0, thunar_icon_factory_changed, factory, NULL);
 
   /* allocate the hash table for the icon cache */
-  factory->icon_cache = g_hash_table_new_full (thunar_icon_key_hash, thunar_icon_key_equal, g_free, g_object_unref);
-
-  /* create a new thumbnailer */
-  factory->thumbnailer = thunar_thumbnailer_new ();
+  factory->icon_cache = g_hash_table_new_full (thunar_icon_key_hash, thunar_icon_key_equal,
+                                               thunar_icon_key_free, g_object_unref);
 }
 
 
@@ -223,10 +212,7 @@ thunar_icon_factory_dispose (GObject *object)
 
   _thunar_return_if_fail (THUNAR_IS_ICON_FACTORY (factory));
 
-  if (G_UNLIKELY (factory->changed_idle_id >= 0))
-    g_source_remove (factory->changed_idle_id);
-
-  if (G_UNLIKELY (factory->sweep_timer_id >= 0))
+  if (G_UNLIKELY (factory->sweep_timer_id != 0))
     g_source_remove (factory->sweep_timer_id);
 
   (*G_OBJECT_CLASS (thunar_icon_factory_parent_class)->dispose) (object);
@@ -238,21 +224,12 @@ static void
 thunar_icon_factory_finalize (GObject *object)
 {
   ThunarIconFactory *factory = THUNAR_ICON_FACTORY (object);
-  guint              n;
 
   _thunar_return_if_fail (THUNAR_IS_ICON_FACTORY (factory));
 
-  /* clear the recently used list */
-  for (n = 0; n < MAX_RECENTLY; ++n)
-    if (G_LIKELY (factory->recently[n] != NULL))
-      g_object_unref (G_OBJECT (factory->recently[n]));
-
   /* clear the icon cache hash table */
   g_hash_table_destroy (factory->icon_cache);
 
-  /* release the thumbnailer */
-  g_object_unref (G_OBJECT (factory->thumbnailer));
-
   /* remove the "changed" emission hook from the GtkIconTheme class */
   g_signal_remove_emission_hook (g_signal_lookup ("changed", GTK_TYPE_ICON_THEME), factory->changed_hook_id);
 
@@ -285,8 +262,8 @@ thunar_icon_factory_get_property (GObject    *object,
       g_value_set_object (value, factory->icon_theme);
       break;
 
-    case PROP_SHOW_THUMBNAILS:
-      g_value_set_boolean (value, factory->show_thumbnails);
+    case PROP_THUMBNAIL_MODE:
+      g_value_set_enum (value, factory->thumbnail_mode);
       break;
 
     default:
@@ -307,8 +284,8 @@ thunar_icon_factory_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_SHOW_THUMBNAILS:
-      factory->show_thumbnails = g_value_get_boolean (value);
+    case PROP_THUMBNAIL_MODE:
+      factory->thumbnail_mode = g_value_get_enum (value);
       break;
 
     default:
@@ -326,18 +303,12 @@ thunar_icon_factory_changed (GSignalInvocationHint *ihint,
                              gpointer               user_data)
 {
   ThunarIconFactory *factory = THUNAR_ICON_FACTORY (user_data);
-  guint              n;
-
-  /* drop all items from the recently used list */
-  for (n = 0; n < MAX_RECENTLY; ++n)
-    if (G_LIKELY (factory->recently[n] != NULL))
-      {
-        g_object_unref (G_OBJECT (factory->recently[n]));
-        factory->recently[n] = NULL;
-      }
 
   /* drop all items from the icon cache */
-  g_hash_table_foreach_remove (factory->icon_cache, (GHRFunc) exo_noop_true, NULL);
+  g_hash_table_remove_all (factory->icon_cache);
+
+  /* bump the stamp so all file icons are reloaded */
+  factory->theme_stamp++;
 
   /* keep the emission hook alive */
   return TRUE;
@@ -376,7 +347,7 @@ thunar_icon_factory_sweep_timer (gpointer user_data)
 static void
 thunar_icon_factory_sweep_timer_destroy (gpointer user_data)
 {
-  THUNAR_ICON_FACTORY (user_data)->sweep_timer_id = -1;
+  THUNAR_ICON_FACTORY (user_data)->sweep_timer_id = 0;
 }
 
 
@@ -453,7 +424,7 @@ thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
        * want to do this for icons displayed in the details view).
        */
       needs_frame = (strstr (path, G_DIR_SEPARATOR_S ".thumbnails" G_DIR_SEPARATOR_S) != NULL)
-                 && (size >= 36) && thumbnail_needs_frame (pixbuf, width, height);
+                 && (size >= 32) && thumbnail_needs_frame (pixbuf, width, height);
 
       /* be sure to make framed thumbnails fit into the size */
       if (G_LIKELY (needs_frame))
@@ -471,7 +442,7 @@ thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
       if (G_LIKELY (width > max_width || height > max_height))
         {
           /* scale down to the required size */
-          tmp = exo_gdk_pixbuf_scale_down (pixbuf, TRUE, max_height, max_height);
+          tmp = exo_gdk_pixbuf_scale_down (pixbuf, TRUE, MAX (1, max_height), MAX (1, max_height));
           g_object_unref (G_OBJECT (pixbuf));
           pixbuf = tmp;
         }
@@ -481,7 +452,7 @@ thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
         {
           /* add a frame to the thumbnail */
           frame = gdk_pixbuf_new_from_inline (-1, thunar_thumbnail_frame, FALSE, NULL);
-          tmp = exo_gdk_pixbuf_frame (pixbuf, frame, 3, 3, 6, 6);
+          tmp = exo_gdk_pixbuf_frame (pixbuf, frame, 4, 3, 5, 6);
           g_object_unref (G_OBJECT (pixbuf));
           g_object_unref (G_OBJECT (frame));
           pixbuf = tmp;
@@ -546,61 +517,27 @@ thunar_icon_factory_lookup_icon (ThunarIconFactory *factory,
         }
 
       /* generate a key for the new cached icon */
-      key = g_malloc (sizeof (ThunarIconKey) + strlen (name) + 1);
-      key->name = ((gchar *) key) + sizeof (ThunarIconKey);
+      key = g_slice_new (ThunarIconKey);
       key->size = size;
-      strcpy (key->name, name);
+      key->name = g_strdup (name);
 
       /* insert the new icon into the cache */
       g_hash_table_insert (factory->icon_cache, key, pixbuf);
     }
 
-  /* add the icon to the recently used list */
-  thunar_icon_factory_mark_recently_used (factory, pixbuf);
+  /* schedule the sweeper */
+  if (G_UNLIKELY (factory->sweep_timer_id == 0))
+    {
+      factory->sweep_timer_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, THUNAR_ICON_FACTORY_SWEEP_TIMEOUT,
+                                                            thunar_icon_factory_sweep_timer, factory,
+                                                            thunar_icon_factory_sweep_timer_destroy);
+    }
 
   return g_object_ref (G_OBJECT (pixbuf));
 }
 
 
 
-static void
-thunar_icon_factory_mark_recently_used (ThunarIconFactory *factory,
-                                        GdkPixbuf         *pixbuf)
-{
-  guint n;
-
-  _thunar_return_if_fail (THUNAR_IS_ICON_FACTORY (factory));
-  _thunar_return_if_fail (GDK_IS_PIXBUF (pixbuf));
-
-  /* check if the icon is already on the list */
-  for (n = 0; n < MAX_RECENTLY; ++n)
-    if (G_UNLIKELY (factory->recently[n] == pixbuf))
-      return;
-
-  /* ditch the previous item on the current insert position,
-   * which - if present - is the oldest item in the list.
-   */
-  if (G_LIKELY (factory->recently[factory->recently_pos] != NULL))
-    g_object_unref (G_OBJECT (factory->recently[factory->recently_pos]));
-
-  /* insert the new pixbuf into the list */
-  factory->recently[factory->recently_pos] = pixbuf;
-  g_object_ref (G_OBJECT (pixbuf));
-
-  /* advance the insert position */
-  factory->recently_pos = (factory->recently_pos + 1) % MAX_RECENTLY;
-
-  /* schedule the sweeper */
-  if (G_UNLIKELY (factory->sweep_timer_id < 0))
-    {
-      factory->sweep_timer_id = g_timeout_add_full (G_PRIORITY_LOW, THUNAR_ICON_FACTORY_SWEEP_TIMEOUT,
-                                                    thunar_icon_factory_sweep_timer, factory,
-                                                    thunar_icon_factory_sweep_timer_destroy);
-    }
-}
-
-
-
 static guint
 thunar_icon_key_hash (gconstpointer data)
 {
@@ -630,7 +567,30 @@ thunar_icon_key_equal (gconstpointer a,
     return FALSE;
 
   /* do a full string comparison on the names */
-  return (strcmp (a_key->name, b_key->name) == 0);
+  return exo_str_is_equal (a_key->name, b_key->name);
+}
+
+
+
+static void
+thunar_icon_key_free (gpointer data)
+{
+  ThunarIconKey *key = data;
+
+  g_free (key->name);
+  g_slice_free (ThunarIconKey, key);
+}
+
+
+
+static void
+thunar_icon_store_free (gpointer data)
+{
+  ThunarIconStore *store = data;
+
+  if (store->icon != NULL)
+    g_object_unref (store->icon);
+  g_slice_free (ThunarIconStore, store);
 }
 
 
@@ -710,7 +670,8 @@ thunar_icon_factory_get_for_icon_theme (GtkIconTheme *icon_theme)
 
       /* connect the "show-thumbnails" property to the global preference */
       factory->preferences = thunar_preferences_get ();
-      exo_mutual_binding_new (G_OBJECT (factory->preferences), "misc-show-thumbnails", G_OBJECT (factory), "show-thumbnails");
+      exo_binding_new (G_OBJECT (factory->preferences), "misc-thumbnail-mode",
+                       G_OBJECT (factory), "thumbnail-mode");
     }
   else
     {
@@ -723,18 +684,41 @@ thunar_icon_factory_get_for_icon_theme (GtkIconTheme *icon_theme)
 
 
 /**
- * thunar_icon_factory_get_icon_theme:
- * @factory : a #ThunarIconFactory instance.
+ * thunar_icon_factory_get_thumbnail_mode:
+ * @factory       : a #ThunarIconFactory instance.
+ * @file          : a #ThunarFile.
  *
- * Returns the #GtkIconTheme associated with @factory.
- *
- * Return value: the #GtkIconTheme associated with @factory.
+ * Return value: if a Thumbnail show be shown for @file.
  **/
-GtkIconTheme*
-thunar_icon_factory_get_icon_theme (const ThunarIconFactory *factory)
+gboolean
+thunar_icon_factory_get_show_thumbnail (const ThunarIconFactory *factory,
+                                        const ThunarFile        *file)
 {
-  _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
-  return factory->icon_theme;
+  GFilesystemPreviewType preview;
+
+  _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), THUNAR_THUMBNAIL_MODE_NEVER);
+  _thunar_return_val_if_fail (file == NULL || THUNAR_IS_FILE (file), THUNAR_THUMBNAIL_MODE_NEVER);
+
+  if (file == NULL
+      || factory->thumbnail_mode == THUNAR_THUMBNAIL_MODE_NEVER)
+    return FALSE;
+
+  /* always create thumbs for local files */
+  if (thunar_file_is_local (file))
+    return TRUE;
+
+  preview = thunar_file_get_preview_type (file);
+
+  /* file system says to never thumbnail anything */
+  if (preview == G_FILESYSTEM_PREVIEW_TYPE_NEVER)
+    return FALSE;
+
+  /* only if the setting is local and the fs reports to be local */
+  if (factory->thumbnail_mode == THUNAR_THUMBNAIL_MODE_ONLY_LOCAL)
+    return preview == G_FILESYSTEM_PREVIEW_TYPE_IF_LOCAL;
+
+  /* THUNAR_THUMBNAIL_MODE_ALWAYS */
+  return TRUE;
 }
 
 
@@ -744,7 +728,6 @@ thunar_icon_factory_get_icon_theme (const ThunarIconFactory *factory)
  * @factory       : a #ThunarIconFactory instance.
  * @name          : name of the icon to load.
  * @size          : desired icon size.
- * @attach_points : location to store the attach points to or %NULL.
  * @wants_default : %TRUE to return the fallback icon if no icon of @name
  *                  is found in the @factory.
  *
@@ -762,7 +745,6 @@ GdkPixbuf*
 thunar_icon_factory_load_icon (ThunarIconFactory        *factory,
                                const gchar              *name,
                                gint                      size,
-                               ThunarEmblemAttachPoints *attach_points,
                                gboolean                  wants_default)
 {
   _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
@@ -771,7 +753,7 @@ thunar_icon_factory_load_icon (ThunarIconFactory        *factory,
   /* cannot happen unless there's no XSETTINGS manager
    * for the default screen, but just in case...
    */
-  if (G_UNLIKELY (name == NULL || *name == '\0'))
+  if (G_UNLIKELY (exo_str_is_empty (name)))
     {
       /* check if the caller will happly accept the fallback icon */
       if (G_LIKELY (wants_default))
@@ -804,46 +786,44 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
                                     ThunarFileIconState icon_state,
                                     gint                icon_size)
 {
-  ThunarFileThumbState thumb_state;
-  GInputStream        *stream;
-  GtkIconInfo         *icon_info;
-  const gchar         *thumbnail_path;
-  GdkPixbuf           *icon = NULL;
-  GIcon               *gicon;
-  gchar               *icon_name;
+  GInputStream    *stream;
+  GtkIconInfo     *icon_info;
+  const gchar     *thumbnail_path;
+  GdkPixbuf       *icon = NULL;
+  GIcon           *gicon;
+  const gchar     *icon_name;
+  const gchar     *custom_icon;
+  ThunarIconStore *store;
 
   _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), NULL);
   _thunar_return_val_if_fail (icon_size > 0, NULL);
 
+  /* check if we have a stored icon on the file and it is still valid */
+  store = g_object_get_qdata (G_OBJECT (file), thunar_icon_factory_store_quark);
+  if (store != NULL
+      && store->icon_state == icon_state
+      && store->icon_size == icon_size
+      && store->stamp == factory->theme_stamp
+      && store->thumb_state == thunar_file_get_thumb_state (file))
+    {
+      return g_object_ref (store->icon);
+    }
+
   /* check if we have a custom icon for this file */
-  icon_name = thunar_file_get_custom_icon (file);
-  if (icon_name != NULL)
+  custom_icon = thunar_file_get_custom_icon (file);
+  if (custom_icon != NULL)
     {
       /* try to load the icon */
-      icon = thunar_icon_factory_lookup_icon (factory, icon_name, icon_size, FALSE);
-      g_free (icon_name);
+      icon = thunar_icon_factory_lookup_icon (factory, custom_icon, icon_size, FALSE);
       if (G_LIKELY (icon != NULL))
         return icon;
     }
 
   /* check if thumbnails are enabled and we can display a thumbnail for the item */
-  if (G_LIKELY (factory->show_thumbnails && thunar_file_is_regular (file)))
+  if (thunar_icon_factory_get_show_thumbnail (factory, file)
+      && thunar_file_is_regular (file))
     {
-      /* this is how thumbnails for files are loaded: first, we check the thumbnail
-       * state. If that is unknown, we request a thumbnail to be generated in the
-       * background. At the same time we already try to load the thumbnail, in case
-       * it's already there. when the thumbnail is ready, we just load it */
-
-      /* determine the thumbnail state of the file */
-      thumb_state = thunar_file_get_thumb_state (file);
-
-      if (thumb_state == THUNAR_FILE_THUMB_STATE_UNKNOWN)
-        {
-          /* we don't know the state yet so request a new thumbnail in the background */
-          thunar_thumbnailer_queue_file (factory->thumbnailer, file);
-        }
-
       /* determine the preview icon first */
       gicon = thunar_file_get_preview_icon (file);
 
@@ -856,7 +836,8 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
               icon_info = 
                 gtk_icon_theme_lookup_by_gicon (factory->icon_theme, 
                                                 gicon, icon_size, 
-                                                GTK_ICON_LOOKUP_USE_BUILTIN);
+                                                GTK_ICON_LOOKUP_USE_BUILTIN
+                                                | GTK_ICON_LOOKUP_FORCE_SIZE);
 
               /* check if the lookup succeeded */
               if (icon_info != NULL)
@@ -876,7 +857,9 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
               if (stream != NULL)
                 {
                   /* load the pixbuf from the stream */
-                  icon = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
+                  icon = gdk_pixbuf_new_from_stream_at_scale (stream, icon_size,
+                                                              icon_size, TRUE,
+                                                              NULL, NULL);
 
                   /* destroy the stream */
                   g_object_unref (stream);
@@ -897,23 +880,48 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
           if (thumbnail_path != NULL)
             {
               /* try to load the thumbnail */
-              icon = thunar_icon_factory_load_from_file (factory, thumbnail_path, 
-                                                         icon_size);
-
-              /* return the thumbnail if it could be loaded */
-              if (icon != NULL)
-                return icon;
+              icon = thunar_icon_factory_load_from_file (factory, thumbnail_path, icon_size);
             }
         }
     }
 
   /* lookup the icon name for the icon in the given state and load the icon */
-  icon_name = thunar_file_get_icon_name (file, icon_state, factory->icon_theme);
-  icon = thunar_icon_factory_load_icon (factory, icon_name, icon_size, NULL, TRUE);
-  g_free (icon_name);
+  if (G_LIKELY (icon == NULL))
+    {
+      icon_name = thunar_file_get_icon_name (file, icon_state, factory->icon_theme);
+      icon = thunar_icon_factory_load_icon (factory, icon_name, icon_size, TRUE);
+    }
+
+  if (G_LIKELY (icon != NULL))
+    {
+      store = g_slice_new (ThunarIconStore);
+      store->icon_size = icon_size;
+      store->icon_state = icon_state;
+      store->stamp = factory->theme_stamp;
+      store->thumb_state = thunar_file_get_thumb_state (file);
+      store->icon = g_object_ref (icon);
+
+      g_object_set_qdata_full (G_OBJECT (file), thunar_icon_factory_store_quark,
+                               store, thunar_icon_store_free);
+    }
+
   return icon;
 }
 
 
 
+/**
+ * thunar_icon_factory_clear_pixmap_cache:
+ * @file : a #ThunarFile.
+ *
+ * Unset the pixmap cache on a file to force a reload on the next request.
+ **/
+void
+thunar_icon_factory_clear_pixmap_cache (ThunarFile *file)
+{
+  _thunar_return_if_fail (THUNAR_IS_FILE (file));
 
+  /* unset the data */
+  if (thunar_icon_factory_store_quark != 0)
+    g_object_set_qdata (G_OBJECT (file), thunar_icon_factory_store_quark, NULL);
+}
