@@ -30,6 +30,7 @@
 #include <thunar/thunar-pango-extensions.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-util.h>
+#include <thunar/thunar-transfer-job.h>
 #include <thunar/thunar-progress-view.h>
 
 
@@ -74,6 +75,9 @@ static void              thunar_progress_view_info_message (ThunarProgressView *
 static void              thunar_progress_view_percent      (ThunarProgressView *view,
                                                             gdouble             percent,
                                                             ExoJob             *job);
+static ThunarJob        *thunar_progress_view_get_job      (ThunarProgressView *view);
+static void              thunar_progress_view_set_job      (ThunarProgressView *view,
+                                                            ThunarJob          *job);
 
 
 
@@ -88,11 +92,9 @@ struct _ThunarProgressView
 
   ThunarJob *job;
 
-  gint64     start_time;
-  gint64     last_update_time;
-
   GtkWidget *progress_bar;
   GtkWidget *progress_label;
+  GtkWidget *message_label;
 
   gchar     *icon_name;
   gchar     *title;
@@ -118,7 +120,7 @@ thunar_progress_view_class_init (ThunarProgressViewClass *klass)
   /**
    * ThunarProgressView:job:
    *
-   * The #ThunarJob, whose progress is displayed by this view, or 
+   * The #ThunarJob, whose progress is displayed by this view, or
    * %NULL if no job is set.
    **/
   g_object_class_install_property (gobject_class,
@@ -174,11 +176,9 @@ thunar_progress_view_init (ThunarProgressView *view)
   GtkWidget *button;
   GtkWidget *vbox;
   GtkWidget *vbox2;
+  GtkWidget *vbox3;
   GtkWidget *hbox;
 
-  /* remember the current time as start time */
-  view->start_time = thunar_util_get_real_time ();
-
   vbox = gtk_vbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (view), vbox);
   gtk_widget_show (vbox);
@@ -189,6 +189,7 @@ thunar_progress_view_init (ThunarProgressView *view)
 
   image = g_object_new (GTK_TYPE_IMAGE, "icon-size", GTK_ICON_SIZE_BUTTON, NULL);
   gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);
+  exo_binding_new (G_OBJECT (view), "icon-name", G_OBJECT (image), "icon-name");
   gtk_widget_show (image);
 
   vbox2 = gtk_vbox_new (FALSE, 6);
@@ -201,26 +202,38 @@ thunar_progress_view_init (ThunarProgressView *view)
   gtk_box_pack_start (GTK_BOX (vbox2), label, TRUE, TRUE, 0);
   gtk_widget_show (label);
 
-  view->progress_label = g_object_new (GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
-  gtk_label_set_ellipsize (GTK_LABEL (view->progress_label), PANGO_ELLIPSIZE_MIDDLE);
-  gtk_box_pack_start (GTK_BOX (vbox2), view->progress_label, TRUE, TRUE, 0);
-  gtk_widget_show (view->progress_label);
+  view->message_label = g_object_new (GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
+  gtk_label_set_ellipsize (GTK_LABEL (view->message_label), PANGO_ELLIPSIZE_MIDDLE);
+  gtk_box_pack_start (GTK_BOX (vbox2), view->message_label, TRUE, TRUE, 0);
+  gtk_widget_show (view->message_label);
 
-  hbox = gtk_hbox_new (FALSE, 12);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+  hbox = gtk_hbox_new (FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
   gtk_widget_show (hbox);
 
-  view->progress_bar = g_object_new (GTK_TYPE_PROGRESS_BAR, "text", " ", NULL);
-  gtk_box_pack_start (GTK_BOX (hbox), view->progress_bar, TRUE, TRUE, 0);
+  vbox3 = gtk_vbox_new (FALSE, 3);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox3, TRUE, TRUE, 0);
+  gtk_widget_show (vbox3);
+
+  view->progress_bar = gtk_progress_bar_new ();
+  gtk_box_pack_start (GTK_BOX (vbox3), view->progress_bar, TRUE, TRUE, 0);
   gtk_widget_show (view->progress_bar);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+  view->progress_label = g_object_new (GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
+  gtk_label_set_ellipsize (GTK_LABEL (view->progress_label), PANGO_ELLIPSIZE_END);
+  gtk_label_set_attributes (GTK_LABEL (view->progress_label), thunar_pango_attr_list_small ());
+  gtk_box_pack_start (GTK_BOX (vbox3), view->progress_label, FALSE, TRUE, 0);
+  gtk_widget_show (view->progress_label);
+
+  button = gtk_button_new ();
   g_signal_connect_swapped (button, "clicked", G_CALLBACK (thunar_progress_view_cancel_job), view);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_widget_set_can_focus (button, FALSE);
   gtk_widget_show (button);
-  
-  /* connect the view icon name to the action image */
-  exo_binding_new (G_OBJECT (view), "icon-name", G_OBJECT (image), "icon-name");
+
+  image = gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON);
+  gtk_container_add (GTK_CONTAINER (button), image);
+  gtk_widget_show (image);
 
   /* connect the view title to the action label */
   exo_binding_new (G_OBJECT (view), "title", G_OBJECT (label), "label");
@@ -330,16 +343,15 @@ thunar_progress_view_cancel_job (ThunarProgressView *view)
       exo_job_cancel (EXO_JOB (view->job));
 
       /* don't listen to percentage updates any more */
-      g_signal_handlers_disconnect_matched (view->job, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, 
+      g_signal_handlers_disconnect_matched (view->job, G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
                                             thunar_progress_view_percent, NULL);
 
       /* don't listen to info messages any more */
       g_signal_handlers_disconnect_matched (view->job, G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
                                             thunar_progress_view_info_message, NULL);
 
-      /* update the progress bar text */
-      gtk_progress_bar_set_text (GTK_PROGRESS_BAR (view->progress_bar), 
-                                 _("Cancelling..."));
+      /* update the status text */
+      gtk_label_set_text (GTK_LABEL (view->progress_label), _("Cancelling..."));
     }
 }
 
@@ -365,7 +377,7 @@ thunar_progress_view_ask (ThunarProgressView *view,
   window = gtk_widget_get_toplevel (GTK_WIDGET (view));
 
   /* display the question view */
-  return thunar_dialogs_show_job_ask (window != NULL ? GTK_WINDOW (window) : NULL, 
+  return thunar_dialogs_show_job_ask (window != NULL ? GTK_WINDOW (window) : NULL,
                                       message, choices);
 }
 
@@ -392,7 +404,7 @@ thunar_progress_view_ask_replace (ThunarProgressView *view,
   window = gtk_widget_get_toplevel (GTK_WIDGET (view));
 
   /* display the question view */
-  return thunar_dialogs_show_job_ask_replace (window != NULL ? GTK_WINDOW (window) : NULL, 
+  return thunar_dialogs_show_job_ask_replace (window != NULL ? GTK_WINDOW (window) : NULL,
                                               src_file, dst_file);
 }
 
@@ -446,7 +458,7 @@ thunar_progress_view_info_message (ThunarProgressView *view,
   _thunar_return_if_fail (THUNAR_IS_JOB (job));
   _thunar_return_if_fail (view->job == THUNAR_JOB (job));
 
-  gtk_label_set_text (GTK_LABEL (view->progress_label), message);
+  gtk_label_set_text (GTK_LABEL (view->message_label), message);
 }
 
 
@@ -456,77 +468,27 @@ thunar_progress_view_percent (ThunarProgressView *view,
                               gdouble             percent,
                               ExoJob             *job)
 {
-  gint64 current_time;
-  gulong remaining_time;
-  gint64 elapsed_time;
-  gchar  text[512];
+  gchar *text;
 
   _thunar_return_if_fail (THUNAR_IS_PROGRESS_VIEW (view));
   _thunar_return_if_fail (percent >= 0.0 && percent <= 100.0);
   _thunar_return_if_fail (THUNAR_IS_JOB (job));
   _thunar_return_if_fail (view->job == THUNAR_JOB (job));
 
+  /* update progressbar */
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (view->progress_bar), percent / 100.0);
 
-  /* check if we should update the time display (every 400ms) */
-  current_time = thunar_util_get_real_time ();
-  if (current_time - view->last_update_time > (400 * 1000))
+  /* set progress text */
+  if (THUNAR_IS_TRANSFER_JOB (job))
     {
-      /* calculate the remaining time (in seconds) */
-      elapsed_time = (current_time - view->start_time) / 1000;
-      remaining_time = ((100 * elapsed_time) / percent - elapsed_time) / 1000;
-
-      /* setup the time label */
-      if (G_LIKELY (remaining_time > 0))
-        {
-          /* format the time text */
-          if (remaining_time > 60 * 60)
-            {
-              remaining_time = (gulong) (remaining_time / (60 * 60));
-              g_snprintf (text, sizeof (text), ngettext ("%lu hour remaining", "%lu hours remaining", remaining_time), remaining_time);
-            }
-          else if (remaining_time > 60)
-            {
-              remaining_time = (gulong) (remaining_time / 60);
-              g_snprintf (text, sizeof (text), ngettext ("%lu minute remaining", "%lu minutes remaining", remaining_time), remaining_time);
-            }
-          else
-            {
-              remaining_time = remaining_time;
-              g_snprintf (text, sizeof (text), ngettext ("%lu second remaining", "%lu seconds remaining", remaining_time), remaining_time);
-            }
-
-          /* apply the time text */
-          gtk_progress_bar_set_text (GTK_PROGRESS_BAR (view->progress_bar), text);
-        }
-      else
-        {
-          /* display an empty label */
-          gtk_progress_bar_set_text (GTK_PROGRESS_BAR (view->progress_bar), " ");
-        }
-
-      /* remember the current time as last update time */
-      view->last_update_time = current_time;
+      text = thunar_transfer_job_get_status (THUNAR_TRANSFER_JOB (job));
+      gtk_label_set_text (GTK_LABEL (view->progress_label), text);
+      g_free (text);
     }
 }
 
 
 
-/**
- * thunar_progress_view_new:
- *
- * Allocates a new #ThunarProgressView.
- *
- * Return value: the newly allocated #ThunarProgressView.
- **/
-GtkWidget*
-thunar_progress_view_new (void)
-{
-  return thunar_progress_view_new_with_job (NULL);
-}
-
-
-
 /**
  * thunar_progress_view_new_with_job:
  * @job : a #ThunarJob or %NULL.
@@ -553,7 +515,7 @@ thunar_progress_view_new_with_job (ThunarJob *job)
  *
  * Return value: the job associated with @view or %NULL.
  **/
-ThunarJob *
+static ThunarJob *
 thunar_progress_view_get_job (ThunarProgressView *view)
 {
   _thunar_return_val_if_fail (THUNAR_IS_PROGRESS_VIEW (view), NULL);
@@ -569,7 +531,7 @@ thunar_progress_view_get_job (ThunarProgressView *view)
  *
  * Associates @job with @view.
  **/
-void
+static void
 thunar_progress_view_set_job (ThunarProgressView *view,
                               ThunarJob          *job)
 {
