@@ -87,6 +87,8 @@ struct _ThunarShortcutsPane
   guint             ui_merge_id;
 
   GtkWidget        *view;
+
+  guint             idle_select_directory;
 };
 
 
@@ -179,6 +181,7 @@ thunar_shortcuts_pane_init (ThunarShortcutsPane *shortcuts_pane)
 
   /* connect the "shortcut-activated" signal */
   g_signal_connect_swapped (G_OBJECT (shortcuts_pane->view), "shortcut-activated", G_CALLBACK (thunar_navigator_change_directory), shortcuts_pane);
+  g_signal_connect_swapped (G_OBJECT (shortcuts_pane->view), "shortcut-activated-tab", G_CALLBACK (thunar_navigator_open_new_tab), shortcuts_pane);
 }
 
 
@@ -282,6 +285,25 @@ thunar_shortcuts_pane_get_current_directory (ThunarNavigator *navigator)
 
 
 
+static gboolean
+thunar_shortcuts_pane_set_current_directory_idle (gpointer data)
+{
+  ThunarShortcutsPane *shortcuts_pane = THUNAR_SHORTCUTS_PANE (data);
+
+  if (shortcuts_pane->current_directory != NULL)
+    {
+      thunar_shortcuts_view_select_by_file (THUNAR_SHORTCUTS_VIEW (shortcuts_pane->view),
+                                            shortcuts_pane->current_directory);
+    }
+
+  /* unset id */
+  shortcuts_pane->idle_select_directory = 0;
+
+  return FALSE;
+}
+
+
+
 static void
 thunar_shortcuts_pane_set_current_directory (ThunarNavigator *navigator,
                                              ThunarFile      *current_directory)
@@ -292,6 +314,13 @@ thunar_shortcuts_pane_set_current_directory (ThunarNavigator *navigator,
   if (G_LIKELY (shortcuts_pane->current_directory != NULL))
     g_object_unref (G_OBJECT (shortcuts_pane->current_directory));
 
+  /* remove pending timeout */
+  if (shortcuts_pane->idle_select_directory != 0)
+    {
+      g_source_remove (shortcuts_pane->idle_select_directory);
+      shortcuts_pane->idle_select_directory = 0;
+    }
+
   /* activate the new directory */
   shortcuts_pane->current_directory = current_directory;
 
@@ -301,8 +330,11 @@ thunar_shortcuts_pane_set_current_directory (ThunarNavigator *navigator,
       /* take a reference on the new directory */
       g_object_ref (G_OBJECT (current_directory));
 
-      /* select the file in the view (if possible) */
-      thunar_shortcuts_view_select_by_file (THUNAR_SHORTCUTS_VIEW (shortcuts_pane->view), current_directory);
+      /* start idle to select item in sidepane (this to also make
+       * the selection work when the bookmarks are loaded idle) */
+      shortcuts_pane->idle_select_directory =
+        g_idle_add_full (G_PRIORITY_LOW, thunar_shortcuts_pane_set_current_directory_idle,
+                         shortcuts_pane, NULL);
     }
 
   /* notify listeners */
@@ -325,16 +357,16 @@ thunar_shortcuts_pane_set_selected_files (ThunarComponent *component,
 {
   ThunarShortcutsPane *shortcuts_pane = THUNAR_SHORTCUTS_PANE (component);
   GtkTreeModel        *model;
-  GtkTreeIter          iter;
+  GtkTreeModel        *child_model;
   GtkAction           *action;
   GList               *lp;
   gint                 n;
 
   /* disconnect from the previously selected files... */
-  thunar_file_list_free (shortcuts_pane->selected_files);
+  thunar_g_file_list_free (shortcuts_pane->selected_files);
 
   /* ...and take a copy of the newly selected files */
-  shortcuts_pane->selected_files = thunar_file_list_copy (selected_files);
+  shortcuts_pane->selected_files = thunar_g_file_list_copy (selected_files);
 
   /* check if the selection contains only folders */
   for (lp = selected_files, n = 0; lp != NULL; lp = lp->next, ++n)
@@ -350,8 +382,9 @@ thunar_shortcuts_pane_set_selected_files (ThunarComponent *component,
       if (G_LIKELY (model != NULL))
         {
           /* check all selected folders */
+          child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
           for (lp = selected_files; lp != NULL; lp = lp->next)
-            if (!thunar_shortcuts_model_iter_for_file (THUNAR_SHORTCUTS_MODEL (model), lp->data, &iter))
+            if (!thunar_shortcuts_model_has_bookmark (THUNAR_SHORTCUTS_MODEL (child_model), thunar_file_get_file (lp->data)))
               break;
         }
 
@@ -436,7 +469,7 @@ thunar_shortcuts_pane_action_shortcuts_add (GtkAction           *action,
                                             ThunarShortcutsPane *shortcuts_pane)
 {
   GtkTreeModel *model;
-  GtkTreePath  *path;
+  GtkTreeModel *child_model;
   GList        *lp;
 
   _thunar_return_if_fail (GTK_IS_ACTION (action));
@@ -447,36 +480,17 @@ thunar_shortcuts_pane_action_shortcuts_add (GtkAction           *action,
   if (G_LIKELY (model != NULL))
     {
       /* add all selected folders to the model */
+      child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
       for (lp = shortcuts_pane->selected_files; lp != NULL; lp = lp->next)
         if (G_LIKELY (thunar_file_is_directory (lp->data)))
           {
             /* append the folder to the shortcuts model */
-            path = gtk_tree_path_new_from_indices (gtk_tree_model_iter_n_children (model, NULL), -1);
-            thunar_shortcuts_model_add (THUNAR_SHORTCUTS_MODEL (model), path, lp->data);
-            gtk_tree_path_free (path);
+            thunar_shortcuts_model_add (THUNAR_SHORTCUTS_MODEL (child_model), NULL, lp->data);
           }
 
       /* update the user interface to reflect the new action state */
-      lp = thunar_file_list_copy (shortcuts_pane->selected_files);
+      lp = thunar_g_file_list_copy (shortcuts_pane->selected_files);
       thunar_component_set_selected_files (THUNAR_COMPONENT (shortcuts_pane), lp);
-      thunar_file_list_free (lp);
+      thunar_g_file_list_free (lp);
     }
 }
-
-
-
-/**
- * thunar_shortcuts_pane_new:
- *
- * Allocates a new #ThunarShortcutsPane instance.
- *
- * Return value: the newly allocated #ThunarShortcutsPane instance.
- **/
-GtkWidget*
-thunar_shortcuts_pane_new (void)
-{
-  return g_object_new (THUNAR_TYPE_SHORTCUTS_PANE, NULL);
-}
-
-
-
