@@ -36,6 +36,14 @@
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-thumbnail-cache.h>
 
+#if GLIB_CHECK_VERSION (2, 32, 0)
+#define _thumbnail_cache_lock(cache)   g_mutex_lock (&((cache)->lock))
+#define _thumbnail_cache_unlock(cache) g_mutex_unlock (&((cache)->lock))
+#else
+#define _thumbnail_cache_lock(cache)   g_mutex_lock ((cache)->lock)
+#define _thumbnail_cache_unlock(cache) g_mutex_unlock ((cache)->lock)
+#endif
+
 
 
 static void thunar_thumbnail_cache_finalize (GObject *object);
@@ -68,8 +76,12 @@ struct _ThunarThumbnailCache
   GList      *cleanup_queue;
   guint       cleanup_queue_idle_id;
 
+#if GLIB_CHECK_VERSION (2, 32, 0)
+  GMutex      lock;
+#else
   GMutex     *lock;
 #endif
+#endif
 };
 
 
@@ -113,9 +125,13 @@ thunar_thumbnail_cache_init (ThunarThumbnailCache *cache)
       dbus_g_connection_unref (connection);
     }
 
-  /* create a new mutex for accessing the cache from different threads */
+/* create a new mutex for accessing the cache from different threads */
+#if GLIB_CHECK_VERSION (2, 32, 0)
+  g_mutex_init (&cache->lock);
+#else
   cache->lock = g_mutex_new ();
 #endif
+#endif
 }
 
 
@@ -127,48 +143,43 @@ thunar_thumbnail_cache_finalize (GObject *object)
   ThunarThumbnailCache *cache = THUNAR_THUMBNAIL_CACHE (object);
 
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
 
   /* drop the move queue idle and all queued files */
   if (cache->move_queue_idle_id > 0)
     g_source_remove (cache->move_queue_idle_id);
-  g_list_foreach (cache->move_source_queue, (GFunc) g_object_unref, NULL);
-  g_list_foreach (cache->move_target_queue, (GFunc) g_object_unref, NULL);
-  g_list_free (cache->move_source_queue);
-  g_list_free (cache->move_target_queue);
+  g_list_free_full (cache->move_source_queue, g_object_unref);
+  g_list_free_full (cache->move_target_queue, g_object_unref);
 
   /* drop the copy queue idle and all queued files */
   if (cache->copy_queue_idle_id > 0)
     g_source_remove (cache->copy_queue_idle_id);
-  g_list_foreach (cache->copy_source_queue, (GFunc) g_object_unref, NULL);
-  g_list_foreach (cache->copy_target_queue, (GFunc) g_object_unref, NULL);
-  g_list_free (cache->copy_source_queue);
-  g_list_free (cache->copy_target_queue);
+  g_list_free_full (cache->copy_source_queue, g_object_unref);
+  g_list_free_full (cache->copy_target_queue, g_object_unref);
 
   /* drop the delete queue idle and all queued files */
   if (cache->delete_queue_idle_id > 0)
     g_source_remove (cache->delete_queue_idle_id);
-  g_list_foreach (cache->delete_queue, (GFunc) g_object_unref, NULL);
-  g_list_free (cache->delete_queue);
+  g_list_free_full (cache->delete_queue, g_object_unref);
 
   /* drop the cleanup queue idle and all queued files */
   if (cache->cleanup_queue_idle_id > 0)
     g_source_remove (cache->cleanup_queue_idle_id);
-  g_list_foreach (cache->cleanup_queue, (GFunc) g_object_unref, NULL);
-  g_list_free (cache->cleanup_queue);
+  g_list_free_full (cache->cleanup_queue, g_object_unref);
 
   /* check if we have a valid cache proxy */
   if (cache->cache_proxy != NULL)
-    {
-      /* release the cache proxy itself */
-      g_object_unref (cache->cache_proxy);
-    }
+    g_object_unref (cache->cache_proxy);
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 
   /* release the mutex itself */
+#if GLIB_CHECK_VERSION (2, 32, 0)
+  g_mutex_clear (&cache->lock);
+#else
   g_mutex_free (cache->lock);
+#endif
 #endif
 
   (*G_OBJECT_CLASS (thunar_thumbnail_cache_parent_class)->finalize) (object);
@@ -297,7 +308,7 @@ thunar_thumbnail_cache_process_move_queue (ThunarThumbnailCache *cache)
   _thunar_return_val_if_fail (THUNAR_IS_THUMBNAIL_CACHE (cache), FALSE);
 
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
   
   /* compute how many URIs there are */
   n_uris = g_list_length (cache->move_source_queue);
@@ -344,7 +355,7 @@ thunar_thumbnail_cache_process_move_queue (ThunarThumbnailCache *cache)
   cache->move_queue_idle_id = 0;
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 
   return FALSE;
 }
@@ -364,7 +375,7 @@ thunar_thumbnail_cache_process_copy_queue (ThunarThumbnailCache *cache)
   _thunar_return_val_if_fail (THUNAR_IS_THUMBNAIL_CACHE (cache), FALSE);
 
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
   
   /* compute how many URIs there are */
   n_uris = g_list_length (cache->copy_source_queue);
@@ -411,7 +422,7 @@ thunar_thumbnail_cache_process_copy_queue (ThunarThumbnailCache *cache)
   cache->copy_queue_idle_id = 0;
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 
   return FALSE;
 }
@@ -429,7 +440,7 @@ thunar_thumbnail_cache_process_delete_queue (ThunarThumbnailCache *cache)
   _thunar_return_val_if_fail (THUNAR_IS_THUMBNAIL_CACHE (cache), FALSE);
 
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
   
   /* compute how many URIs there are */
   n_uris = g_list_length (cache->delete_queue);
@@ -463,7 +474,7 @@ thunar_thumbnail_cache_process_delete_queue (ThunarThumbnailCache *cache)
   cache->delete_queue_idle_id = 0;
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 
   return FALSE;
 }
@@ -481,7 +492,7 @@ thunar_thumbnail_cache_process_cleanup_queue (ThunarThumbnailCache *cache)
   _thunar_return_val_if_fail (THUNAR_IS_THUMBNAIL_CACHE (cache), FALSE);
 
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
   
   /* compute how many URIs there are */
   n_uris = g_list_length (cache->cleanup_queue);
@@ -523,7 +534,7 @@ thunar_thumbnail_cache_process_cleanup_queue (ThunarThumbnailCache *cache)
   cache->cleanup_queue_idle_id = 0;
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 
   return FALSE;
 }
@@ -550,7 +561,7 @@ thunar_thumbnail_cache_move_file (ThunarThumbnailCache *cache,
 
 #ifdef HAVE_DBUS
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
 
   /* check if we have a valid proxy for the cache service */
   if (cache->cache_proxy != NULL)
@@ -575,7 +586,7 @@ thunar_thumbnail_cache_move_file (ThunarThumbnailCache *cache,
     }
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 #endif
 }
 
@@ -592,7 +603,7 @@ thunar_thumbnail_cache_copy_file (ThunarThumbnailCache *cache,
 
 #ifdef HAVE_DBUS
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
 
   /* check if we have a valid proxy for the cache service */
   if (cache->cache_proxy != NULL)
@@ -617,7 +628,7 @@ thunar_thumbnail_cache_copy_file (ThunarThumbnailCache *cache,
     }
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 #endif
 }
 
@@ -632,7 +643,7 @@ thunar_thumbnail_cache_delete_file (ThunarThumbnailCache *cache,
 
 #ifdef HAVE_DBUS
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
 
   /* check if we have a valid proxy for the cache service */
   if (cache->cache_proxy)
@@ -654,7 +665,7 @@ thunar_thumbnail_cache_delete_file (ThunarThumbnailCache *cache,
     }
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 #endif
 }
 
@@ -669,7 +680,7 @@ thunar_thumbnail_cache_cleanup_file (ThunarThumbnailCache *cache,
 
 #ifdef HAVE_DBUS
   /* acquire a cache lock */
-  g_mutex_lock (cache->lock);
+  _thumbnail_cache_lock (cache);
 
   /* check if we have a valid proxy for the cache service */
   if (cache->cache_proxy)
@@ -691,6 +702,6 @@ thunar_thumbnail_cache_cleanup_file (ThunarThumbnailCache *cache,
     }
 
   /* release the cache lock */
-  g_mutex_unlock (cache->lock);
+  _thumbnail_cache_unlock (cache);
 #endif
 }
