@@ -1,22 +1,22 @@
-/* vi:set sw=2 sts=2 ts=2 et ai: */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify 
- * it under the terms of the GNU General Public License as published by 
- * the Free Software Foundation; either version 2 of the License, or (at 
- * your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
- * General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
- * MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the Free
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,15 +25,22 @@
 
 #include <gio/gio.h>
 
+#include <thunar/thunar-application.h>
 #include <thunar/thunar-gio-extensions.h>
 #include <thunar/thunar-io-scan-directory.h>
 #include <thunar/thunar-io-jobs-util.h>
 #include <thunar/thunar-job.h>
 #include <thunar/thunar-private.h>
+#include <thunar/thunar-thumbnail-cache.h>
 #include <thunar/thunar-transfer-job.h>
 
 
 
+/* seconds before we show the transfer rate + remaining time */
+#define MINIMUM_TRANSFER_TIME (10 * G_USEC_PER_SEC) /* 10 seconds */
+
+
+
 typedef struct _ThunarTransferNode ThunarTransferNode;
 
 
@@ -41,7 +48,7 @@ typedef struct _ThunarTransferNode ThunarTransferNode;
 static void     thunar_transfer_job_finalize     (GObject                *object);
 static gboolean thunar_transfer_job_execute      (ExoJob                 *job,
                                                   GError                **error);
-static void     thunar_transfer_node_free        (ThunarTransferNode     *node);
+static void     thunar_transfer_node_free        (gpointer                data);
 
 
 
@@ -58,11 +65,14 @@ struct _ThunarTransferJob
   GList                *source_node_list;
   GList                *target_file_list;
 
+  gint64                start_time;
+  gint64                last_update_time;
+  guint64               last_total_progress;
+
   guint64               total_size;
   guint64               total_progress;
   guint64               file_progress;
-
-  gdouble               previous_percentage;
+  guint64               transfer_rate;
 };
 
 struct _ThunarTransferNode
@@ -85,7 +95,7 @@ thunar_transfer_job_class_init (ThunarTransferJobClass *klass)
   ExoJobClass  *exojob_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
-  gobject_class->finalize = thunar_transfer_job_finalize; 
+  gobject_class->finalize = thunar_transfer_job_finalize;
 
   exojob_class = EXO_JOB_CLASS (klass);
   exojob_class->execute = thunar_transfer_job_execute;
@@ -102,7 +112,10 @@ thunar_transfer_job_init (ThunarTransferJob *job)
   job->total_size = 0;
   job->total_progress = 0;
   job->file_progress = 0;
-  job->previous_percentage = 0.0;
+  job->last_update_time = 0;
+  job->last_total_progress = 0;
+  job->transfer_rate = 0;
+  job->start_time = 0;
 }
 
 
@@ -112,8 +125,7 @@ thunar_transfer_job_finalize (GObject *object)
 {
   ThunarTransferJob *job = THUNAR_TRANSFER_JOB (object);
 
-  g_list_foreach (job->source_node_list, (GFunc) thunar_transfer_node_free, NULL);
-  g_list_free (job->source_node_list);
+  g_list_free_full (job->source_node_list, thunar_transfer_node_free);
 
   thunar_g_file_list_free (job->target_file_list);
 
@@ -127,12 +139,14 @@ thunar_transfer_job_progress (goffset  current_num_bytes,
                               goffset  total_num_bytes,
                               gpointer user_data)
 {
-  guint64 new_percentage;
-
   ThunarTransferJob *job = user_data;
+  guint64            new_percentage;
+  gint64             current_time;
+  gint64             expired_time;
+  guint64            transfer_rate;
 
   _thunar_return_if_fail (THUNAR_IS_TRANSFER_JOB (job));
-  
+
   if (G_LIKELY (job->total_size > 0))
     {
       /* update total progress */
@@ -144,15 +158,28 @@ thunar_transfer_job_progress (goffset  current_num_bytes,
       /* compute the new percentage after the progress we've made */
       new_percentage = (job->total_progress * 100.0) / job->total_size;
 
-      /* notify callers about the progress only if we have advanced by
-       * at least 0.01 percent since the last signal emission */
-      if (new_percentage >= (job->previous_percentage + 0.01))
+      /* get current time */
+      current_time = g_get_real_time ();
+      expired_time = current_time - job->last_update_time;
+
+      /* notify callers not more then every 500ms */
+      if (expired_time > (500 * 1000))
         {
+          /* calculate the transfer rate in the last expired time */
+          transfer_rate = (job->total_progress - job->last_total_progress) / ((gfloat) expired_time / G_USEC_PER_SEC);
+
+          /* take the average of the last 10 rates (5 sec), so the output is less jumpy */
+          if (job->transfer_rate > 0)
+            job->transfer_rate = ((job->transfer_rate * 10) + transfer_rate) / 11;
+          else
+            job->transfer_rate = transfer_rate;
+
           /* emit the percent signal */
           exo_job_percent (EXO_JOB (job), new_percentage);
 
-          /* remember the percentage */
-          job->previous_percentage = new_percentage;
+          /* update internals */
+          job->last_update_time = current_time;
+          job->last_total_progress = job->total_progress;
         }
     }
 }
@@ -177,7 +204,7 @@ thunar_transfer_job_collect_node (ThunarTransferJob  *job,
   if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
     return FALSE;
 
-  info = g_file_query_info (node->source_file, 
+  info = g_file_query_info (node->source_file,
                             G_FILE_ATTRIBUTE_STANDARD_SIZE ","
                             G_FILE_ATTRIBUTE_STANDARD_TYPE,
                             G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
@@ -195,7 +222,7 @@ thunar_transfer_job_collect_node (ThunarTransferJob  *job,
       /* scan the directory for immediate children */
       file_list = thunar_io_scan_directory (THUNAR_JOB (job), node->source_file,
                                             G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                            FALSE, FALSE, &err);
+                                            FALSE, FALSE, FALSE, &err);
 
       /* add children to the transfer node */
       for (lp = file_list; err == NULL && lp != NULL; lp = lp->next)
@@ -211,7 +238,7 @@ thunar_transfer_job_collect_node (ThunarTransferJob  *job,
           /* collect the child node */
           thunar_transfer_job_collect_node (job, child_node, &err);
         }
-      
+
       /* release the child files */
       thunar_g_file_list_free (file_list);
     }
@@ -285,12 +312,12 @@ ttj_copy_file (ThunarTransferJob *job,
   /* check if there were errors */
   if (G_UNLIKELY (err != NULL && err->domain == G_IO_ERROR))
     {
-      if (err->code == G_IO_ERROR_WOULD_MERGE 
-          || (err->code == G_IO_ERROR_EXISTS 
+      if (err->code == G_IO_ERROR_WOULD_MERGE
+          || (err->code == G_IO_ERROR_EXISTS
               && source_type == G_FILE_TYPE_DIRECTORY
               && target_type == G_FILE_TYPE_DIRECTORY))
         {
-          /* we tried to overwrite a directory with a directory. this normally results 
+          /* we tried to overwrite a directory with a directory. this normally results
            * in a merge. ignore the error if we actually *want* to merge */
           if (merge_directories)
             g_clear_error (&err);
@@ -299,14 +326,14 @@ ttj_copy_file (ThunarTransferJob *job,
         {
           g_clear_error (&err);
 
-          /* we tried to copy a directory and either 
+          /* we tried to copy a directory and either
            *
-           * - the target did not exist which means we simple have to 
+           * - the target did not exist which means we simple have to
            *   create the target directory
            *
            * or
            *
-           * - the target is not a directory and we tried to overwrite it in 
+           * - the target is not a directory and we tried to overwrite it in
            *   which case we have to delete it first and then create the target
            *   directory
            */
@@ -321,8 +348,8 @@ ttj_copy_file (ThunarTransferJob *job,
               if (target_exists)
                 {
                   /* the target still exists and thus is not a directory. try to remove it */
-                  g_file_delete (target_file, 
-                                 exo_job_get_cancellable (EXO_JOB (job)), 
+                  g_file_delete (target_file,
+                                 exo_job_get_cancellable (EXO_JOB (job)),
                                  &err);
                 }
 
@@ -330,8 +357,8 @@ ttj_copy_file (ThunarTransferJob *job,
               if (err == NULL)
                 {
                   /* now try to create the directory */
-                  g_file_make_directory (target_file, 
-                                         exo_job_get_cancellable (EXO_JOB (job)), 
+                  g_file_make_directory (target_file,
+                                         exo_job_get_cancellable (EXO_JOB (job)),
                                          &err);
                 }
             }
@@ -361,16 +388,16 @@ ttj_copy_file (ThunarTransferJob *job,
  * Tries to copy @source_file to @target_file. The real destination is the
  * return value and may differ from @target_file (e.g. if you try to copy
  * the file "/foo/bar" into the same directory you'll end up with something
- * like "/foo/copy of bar" instead of "/foo/bar". 
+ * like "/foo/copy of bar" instead of "/foo/bar".
  *
  * The return value is guaranteed to be %NULL on errors and @error will
  * always be set in those cases. If the file is skipped, the return value
  * will be @source_file.
  *
- * Return value: the destination #GFile to which @source_file was copied 
- *               or linked. The caller is reposible to release it with 
- *               g_object_unref() if no longer needed. It points to 
- *               @source_file if the file was skipped and will be %NULL 
+ * Return value: the destination #GFile to which @source_file was copied
+ *               or linked. The caller is reposible to release it with
+ *               g_object_unref() if no longer needed. It points to
+ *               @source_file if the file was skipped and will be %NULL
  *               on error or cancellation.
  **/
 static GFile *
@@ -410,8 +437,8 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
           for (n = 1; err == NULL; ++n)
             {
               GFile *duplicate_file = thunar_io_jobs_util_next_duplicate_file (THUNAR_JOB (job),
-                                                                               source_file, 
-                                                                               TRUE, n, 
+                                                                               source_file,
+                                                                               TRUE, n,
                                                                                &err);
 
               if (err == NULL)
@@ -422,7 +449,7 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
                       /* return the real target file */
                       return duplicate_file;
                     }
-                  
+
                   g_object_unref (duplicate_file);
                 }
 
@@ -441,7 +468,7 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
           g_clear_error (&err);
 
           /* ask the user whether to replace the target file */
-          response = thunar_job_ask_replace (THUNAR_JOB (job), source_file, 
+          response = thunar_job_ask_replace (THUNAR_JOB (job), source_file,
                                              target_file, &err);
 
           if (err != NULL)
@@ -458,7 +485,7 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
               continue;
             }
 
-          /* tell the caller we skipped the file if the user 
+          /* tell the caller we skipped the file if the user
            * doesn't want to retry/overwrite */
           if (response == THUNAR_JOB_RESPONSE_NO)
             return g_object_ref (source_file);
@@ -481,11 +508,13 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
                                GList             **target_file_list_return,
                                GError            **error)
 {
-  ThunarJobResponse response;
-  GFileInfo        *info;
-  GError           *err = NULL;
-  GFile            *real_target_file = NULL;
-  gchar            *base_name;
+  ThunarThumbnailCache *thumbnail_cache;
+  ThunarApplication    *application;
+  ThunarJobResponse     response;
+  GFileInfo            *info;
+  GError               *err = NULL;
+  GFile                *real_target_file = NULL;
+  gchar                *base_name;
 
   _thunar_return_if_fail (THUNAR_IS_TRANSFER_JOB (job));
   _thunar_return_if_fail (node != NULL && G_IS_FILE (node->source_file));
@@ -498,6 +527,11 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
    * wrt restoring files from the trash. Other transfer_nodes will be called with target_parent_file.
    */
 
+  /* take a reference on the thumbnail cache */
+  application = thunar_application_get ();
+  thumbnail_cache = thunar_application_get_thumbnail_cache (application);
+  g_object_unref (application);
+
   for (; err == NULL && node != NULL; node = node->next)
     {
       /* guess the target file for this node (unless already provided) */
@@ -529,13 +563,18 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
 
 retry_copy:
       /* copy the item specified by this node (not recursively) */
-      real_target_file = thunar_transfer_job_copy_file (job, node->source_file, 
+      real_target_file = thunar_transfer_job_copy_file (job, node->source_file,
                                                         target_file, &err);
       if (G_LIKELY (real_target_file != NULL))
         {
           /* node->source_file == real_target_file means to skip the file */
           if (G_LIKELY (node->source_file != real_target_file))
             {
+              /* notify the thumbnail cache of the copy operation */
+              thunar_thumbnail_cache_copy_file (thumbnail_cache,
+                                                node->source_file,
+                                                real_target_file);
+
               /* check if we have children to copy */
               if (node->children != NULL)
                 {
@@ -558,31 +597,46 @@ retry_copy:
 
               /* add the real target file to the return list */
               if (G_LIKELY (target_file_list_return != NULL))
-                *target_file_list_return = thunar_g_file_list_prepend (*target_file_list_return, real_target_file);
+                {
+                  *target_file_list_return =
+                    thunar_g_file_list_prepend (*target_file_list_return,
+                                                real_target_file);
+                }
 
 retry_remove:
               /* try to remove the source directory if we are on copy+remove fallback for move */
-              if (job->type == THUNAR_TRANSFER_JOB_MOVE && 
-                  !g_file_delete (node->source_file, exo_job_get_cancellable (EXO_JOB (job)), &err))
+              if (job->type == THUNAR_TRANSFER_JOB_MOVE)
                 {
-                  /* ask the user to retry */
-                  response = thunar_job_ask_skip (THUNAR_JOB (job), "%s", err->message);
+                  if (g_file_delete (node->source_file,
+                                     exo_job_get_cancellable (EXO_JOB (job)),
+                                     &err))
+                    {
+                      /* notify the thumbnail cache of the delete operation */
+                      thunar_thumbnail_cache_delete_file (thumbnail_cache,
+                                                          node->source_file);
+                    }
+                  else
+                    {
+                      /* ask the user to retry */
+                      response = thunar_job_ask_skip (THUNAR_JOB (job), "%s",
+                                                      err->message);
 
-                  /* reset the error */
-                  g_clear_error (&err);
+                      /* reset the error */
+                      g_clear_error (&err);
 
-                  /* check whether to retry */
-                  if (G_UNLIKELY (response == THUNAR_JOB_RESPONSE_RETRY))
-                    goto retry_remove;
+                      /* check whether to retry */
+                      if (G_UNLIKELY (response == THUNAR_JOB_RESPONSE_RETRY))
+                        goto retry_remove;
+                    }
                 }
             }
 
           g_object_unref (real_target_file);
         }
       else if (err != NULL)
-        { 
+        {
           /* we can only skip if there is space left on the device */
-          if (err->domain != G_IO_ERROR || err->code != G_IO_ERROR_NO_SPACE) 
+          if (err->domain != G_IO_ERROR || err->code != G_IO_ERROR_NO_SPACE)
             {
               /* ask the user to skip this node and all subnodes */
               response = thunar_job_ask_skip (THUNAR_JOB (job), "%s", err->message);
@@ -604,30 +658,125 @@ retry_remove:
       g_object_unref (info);
     }
 
+  /* release the thumbnail cache */
+  g_object_unref (thumbnail_cache);
+
   /* propagate error if we failed or the job was cancelled */
   if (G_UNLIKELY (err != NULL))
     g_propagate_error (error, err);
 }
 
 
+
+static gboolean
+thunar_transfer_job_veryify_destination (ThunarTransferJob  *transfer_job,
+                                         GError            **error)
+{
+  GFileInfo *filesystem_info;
+  guint64     free_space;
+  GFile     *dest;
+  GFileInfo *dest_info;
+  gchar     *dest_name = NULL;
+  gchar     *base_name;
+  gboolean   succeed = TRUE;
+  gchar     *size_string;
+
+  _thunar_return_val_if_fail (THUNAR_IS_TRANSFER_JOB (transfer_job), FALSE);
+
+  /* no target file list */
+  if (transfer_job->target_file_list == NULL)
+    return TRUE;
+
+  /* total size is nul, should be fine */
+  if (transfer_job->total_size == 0)
+    return TRUE;
+
+  /* for all actions in thunar use the same target directory so
+   * although not all files are checked, this should work nicely */
+  dest = g_file_get_parent (G_FILE (transfer_job->target_file_list->data));
+
+  /* query information about the filesystem */
+  filesystem_info = g_file_query_filesystem_info (dest, THUNARX_FILESYSTEM_INFO_NAMESPACE,
+                                                  exo_job_get_cancellable (EXO_JOB (transfer_job)),
+                                                  NULL);
+
+  /* unable to query the info, this could happen on some backends */
+  if (filesystem_info == NULL)
+    {
+      g_object_unref (G_OBJECT (dest));
+      return TRUE;
+    }
+
+  /* some info about the file */
+  dest_info = g_file_query_info (dest, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, 0,
+                                 exo_job_get_cancellable (EXO_JOB (transfer_job)),
+                                 NULL);
+  if (dest_info != NULL)
+    {
+      dest_name = g_strdup (g_file_info_get_display_name (dest_info));
+      g_object_unref (G_OBJECT (dest_info));
+    }
+
+  if (dest_name == NULL)
+    {
+      base_name = g_file_get_basename (dest);
+      dest_name = g_filename_display_name (base_name);
+      g_free (base_name);
+    }
+
+  if (g_file_info_has_attribute (filesystem_info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE))
+    {
+      free_space = g_file_info_get_attribute_uint64 (filesystem_info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+      if (transfer_job->total_size > free_space)
+        {
+          size_string = g_format_size (transfer_job->total_size - free_space);
+          succeed = thunar_job_ask_no_size (THUNAR_JOB (transfer_job),
+                                             _("Error while copying to \"%s\": %s more space is "
+                                               "required to copy to the destination"),
+                                            dest_name, size_string);
+          g_free (size_string);
+        }
+    }
+
+  if (succeed && g_file_info_get_attribute_boolean (filesystem_info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY))
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_READ_ONLY,
+                   _("Error while copying to \"%s\": The destination is read-only"),
+                   dest_name);
+
+      /* meh */
+      succeed = FALSE;
+    }
+
+  g_object_unref (filesystem_info);
+  g_object_unref (G_OBJECT (dest));
+  g_free (dest_name);
+
+  return succeed;
+}
+
+
+
 static gboolean
 thunar_transfer_job_execute (ExoJob  *job,
                              GError **error)
 {
-  ThunarTransferNode *node;
-  ThunarJobResponse   response;
-  ThunarTransferJob  *transfer_job = THUNAR_TRANSFER_JOB (job);
-  GFileInfo          *info;
-  gboolean            parent_exists;
-  GError             *err = NULL;
-  GList              *new_files_list = NULL;
-  GList              *snext;
-  GList              *sp;
-  GList              *tnext;
-  GList              *tp;
-  GFile              *target_parent;
-  gchar              *base_name;
-  gchar              *parent_display_name;
+  ThunarThumbnailCache *thumbnail_cache;
+  ThunarTransferNode   *node;
+  ThunarApplication    *application;
+  ThunarJobResponse     response;
+  ThunarTransferJob    *transfer_job = THUNAR_TRANSFER_JOB (job);
+  GFileInfo            *info;
+  gboolean              parent_exists;
+  GError               *err = NULL;
+  GList                *new_files_list = NULL;
+  GList                *snext;
+  GList                *sp;
+  GList                *tnext;
+  GList                *tp;
+  GFile                *target_parent;
+  gchar                *base_name;
+  gchar                *parent_display_name;
 
   _thunar_return_val_if_fail (THUNAR_IS_TRANSFER_JOB (job), FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -637,6 +786,11 @@ thunar_transfer_job_execute (ExoJob  *job,
 
   exo_job_info_message (job, _("Collecting files..."));
 
+  /* take a reference on the thumbnail cache */
+  application = thunar_application_get ();
+  thumbnail_cache = thunar_application_get_thumbnail_cache (application);
+  g_object_unref (application);
+
   for (sp = transfer_job->source_node_list, tp = transfer_job->target_file_list;
        sp != NULL && tp != NULL && err == NULL;
        sp = snext, tp = tnext)
@@ -658,7 +812,7 @@ thunar_transfer_job_execute (ExoJob  *job,
         break;
 
       /* check if we are moving a file out of the trash */
-      if (transfer_job->type == THUNAR_TRANSFER_JOB_MOVE 
+      if (transfer_job->type == THUNAR_TRANSFER_JOB_MOVE
           && thunar_g_file_is_trashed (node->source_file))
         {
           /* update progress information */
@@ -693,7 +847,7 @@ thunar_transfer_job_execute (ExoJob  *job,
                                                 _("The folder \"%s\" does not exist anymore but is "
                                                   "required to restore the file \"%s\" from the "
                                                   "trash"),
-                                                parent_display_name, 
+                                                parent_display_name,
                                                 g_file_info_get_display_name (info));
 
               /* abort if cancelled */
@@ -705,7 +859,7 @@ thunar_transfer_job_execute (ExoJob  *job,
                 }
 
               /* try to create the parent directory */
-              if (!g_file_make_directory_with_parents (target_parent, 
+              if (!g_file_make_directory_with_parents (target_parent,
                                                        exo_job_get_cancellable (job),
                                                        &err))
                 {
@@ -715,7 +869,7 @@ thunar_transfer_job_execute (ExoJob  *job,
 
                       /* overwrite the internal GIO error with something more user-friendly */
                       g_set_error (&err, G_IO_ERROR, G_IO_ERROR_FAILED,
-                                   _("Failed to restore the folder \"%s\""), 
+                                   _("Failed to restore the folder \"%s\""),
                                    parent_display_name);
                     }
 
@@ -731,19 +885,24 @@ thunar_transfer_job_execute (ExoJob  *job,
           if (target_parent != NULL)
             g_object_unref (target_parent);
         }
-      
+
       if (transfer_job->type == THUNAR_TRANSFER_JOB_MOVE)
         {
           /* update progress information */
-          exo_job_info_message (job, _("Trying to move \"%s\""), 
+          exo_job_info_message (job, _("Trying to move \"%s\""),
                                 g_file_info_get_display_name (info));
 
-          if (g_file_move (node->source_file, tp->data, 
-                           G_FILE_COPY_NOFOLLOW_SYMLINKS 
+          if (g_file_move (node->source_file, tp->data,
+                           G_FILE_COPY_NOFOLLOW_SYMLINKS
                            | G_FILE_COPY_NO_FALLBACK_FOR_MOVE,
                            exo_job_get_cancellable (job),
                            NULL, NULL, &err))
             {
+              /* notify the thumbnail cache of the move operation */
+              thunar_thumbnail_cache_move_file (thumbnail_cache,
+                                                node->source_file,
+                                                tp->data);
+
               /* add the target file to the new files list */
               new_files_list = thunar_g_file_list_prepend (new_files_list, tp->data);
 
@@ -781,15 +940,36 @@ thunar_transfer_job_execute (ExoJob  *job,
       g_object_unref (info);
     }
 
+  /* release the thumbnail cache */
+  g_object_unref (thumbnail_cache);
+
   /* continue if there were no errors yet */
   if (G_LIKELY (err == NULL))
     {
+      /* check destination */
+      if (!thunar_transfer_job_veryify_destination (transfer_job, &err))
+        {
+          if (err != NULL)
+            {
+              g_propagate_error (error, err);
+              return FALSE;
+            }
+          else
+            {
+              /* pretend nothing happened */
+              return TRUE;
+            }
+        }
+
+      /* transfer starts now */
+      transfer_job->start_time = g_get_real_time ();
+
       /* perform the copy recursively for all source transfer nodes */
       for (sp = transfer_job->source_node_list, tp = transfer_job->target_file_list;
            sp != NULL && tp != NULL && err == NULL;
            sp = sp->next, tp = tp->next)
         {
-          thunar_transfer_job_copy_node (transfer_job, sp->data, tp->data, NULL, 
+          thunar_transfer_job_copy_node (transfer_job, sp->data, tp->data, NULL,
                                          &new_files_list, &err);
         }
     }
@@ -811,8 +991,9 @@ thunar_transfer_job_execute (ExoJob  *job,
 
 
 static void
-thunar_transfer_node_free (ThunarTransferNode *node)
+thunar_transfer_node_free (gpointer data)
 {
+  ThunarTransferNode *node = data;
   ThunarTransferNode *next;
 
   /* free all nodes in a row */
@@ -855,8 +1036,8 @@ thunar_transfer_job_new (GList                *source_node_list,
   job->type = type;
 
   /* add a transfer node for each source path and a matching target parent path */
-  for (sp = source_node_list, tp = target_file_list; 
-       sp != NULL; 
+  for (sp = source_node_list, tp = target_file_list;
+       sp != NULL;
        sp = sp->next, tp = tp->next)
     {
       /* make sure we don't transfer root directories. this should be prevented in the GUI */
@@ -881,3 +1062,70 @@ thunar_transfer_job_new (GList                *source_node_list,
 
   return THUNAR_JOB (job);
 }
+
+
+
+gchar *
+thunar_transfer_job_get_status (ThunarTransferJob *job)
+{
+  gchar   *total_size_str;
+  gchar   *total_progress_str;
+  gchar   *transfer_rate_str;
+  GString *status;
+  gulong   remaining_time;
+
+  _thunar_return_val_if_fail (THUNAR_IS_TRANSFER_JOB (job), NULL);
+
+  status = g_string_sized_new (100);
+
+  /* transfer status like "22.6MB of 134.1MB" */
+  total_size_str = g_format_size (job->total_size);
+  total_progress_str = g_format_size (job->total_progress);
+  g_string_append_printf (status, _("%s of %s"), total_progress_str, total_size_str);
+  g_free (total_size_str);
+  g_free (total_progress_str);
+
+  /* show time and transfer rate after 10 seconds */
+  if (job->transfer_rate > 0
+      && (job->last_update_time - job->start_time) > MINIMUM_TRANSFER_TIME)
+    {
+      /* remaining time based on the transfer speed */
+      transfer_rate_str = g_format_size (job->transfer_rate);
+      remaining_time = (job->total_size - job->total_progress) / job->transfer_rate;
+
+      if (remaining_time > 0)
+        {
+          /* insert long dash */
+          g_string_append (status, " \xE2\x80\x94 ");
+
+          if (remaining_time > 60 * 60)
+            {
+              remaining_time = (gulong) (remaining_time / (60 * 60));
+              g_string_append_printf (status, ngettext ("%lu hour remaining (%s/sec)",
+                                                        "%lu hours remaining (%s/sec)",
+                                                        remaining_time),
+                                                        remaining_time, transfer_rate_str);
+            }
+          else if (remaining_time > 60)
+            {
+              remaining_time = (gulong) (remaining_time / 60);
+              g_string_append_printf (status, ngettext ("%lu minute remaining (%s/sec)",
+                                                        "%lu minutes remaining (%s/sec)",
+                                                        remaining_time),
+                                                        remaining_time, transfer_rate_str);
+            }
+          else
+            {
+              g_string_append_printf (status, ngettext ("%lu second remaining (%s/sec)",
+                                                        "%lu seconds remaining (%s/sec)",
+                                                        remaining_time),
+                                                        remaining_time, transfer_rate_str);
+            }
+        }
+
+      g_free (transfer_rate_str);
+    }
+
+  return g_string_free (status, FALSE);
+}
+
